package com.gic.store.service.outer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.store.dao.mapper.TabStoreGroupMapper;
import com.gic.store.dto.*;
import com.gic.store.entity.*;
import com.gic.store.service.*;
import com.gic.store.utils.ErrorCode;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/6/27
 */
@Service("storeApiService")
@SuppressWarnings("unchecked")
public class StoreApiServiceImpl implements StoreApiService {

    @Autowired
    private StoreService storeService;
    @Autowired
    private StoreBrandService storeBrandService;
    @Autowired
    private StorePhotoService storePhotoService;
    @Autowired
    private StoreBusinessTimeService storeBusinessTimeService;
    @Autowired
    private TabStoreGroupMapper tabStoreGroupMapper;
    @Autowired
    private StoreExtendService storeExtendService;
    @Autowired
    private StoreFieldService storeFieldService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Integer> saveOrUpdate(StoreDTO storeDTO) {

        Integer count = storeService.countByStoreCode(storeDTO.getEnterpriseId(), storeDTO.getStoreCode(), storeDTO.getStoreId());
        if (count > 0) {
            return ServiceResponse.failure(ErrorCode.ERR_6.getCode(), "关联主键参数已经存在");
        }
        Integer completeStatus = storeService.checkCompleteStatus(storeDTO);
        storeDTO.setCompleteStatus(completeStatus);
        // 门店自身信息
        if (storeDTO.getStoreId() == null) {
            Integer id = storeService.save(storeDTO);
            storeDTO.setStoreId(id);
        } else {
            storeService.update(storeDTO);
        }
        // 自定义属性
        storeExtendService.saveOrUpdate(storeDTO.getEnterpriseId(), storeDTO.getStoreId(), storeDTO.getStoreExtendList());
        // 门店图片
        List<Integer> delPhotos = null;
        if (StringUtils.isNotEmpty(storeDTO.getDelPhotos())) {
            List<String> photoIds = Arrays.asList(storeDTO.getDelPhotos().split(","));
            delPhotos = photoIds.stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        storePhotoService.editStoreImage(storeDTO.getEnterpriseId(), storeDTO.getStoreId(), storeDTO.getPhotoList(), delPhotos);
        // 营业时间
        List<Integer> delBusinessTimes = null;
        if (StringUtils.isNotEmpty(storeDTO.getDelBusinessTimes())) {
            List<String> delBusinessTimeIds = Arrays.asList(storeDTO.getDelBusinessTimes().split(","));
            delBusinessTimes = delBusinessTimeIds.stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        storeBusinessTimeService.editStoreBusinessTime(storeDTO.getEnterpriseId(), storeDTO.getStoreId(), storeDTO.getBusinessTimeList(), delBusinessTimes);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<StoreDTO>> listStore(StoreDTO storeDTO, Integer pageNum, Integer pageSize) {
        // TODO es 暂时先查询数据库
        com.github.pagehelper.Page storeList = storeService.listStore(storeDTO, pageNum, pageSize);
        Page<StoreDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeList, StoreDTO.class);
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<StoreDTO> getStoreById(Integer enterpriseId, Integer storeId) {
        TabStore store = storeService.getById(storeId);
        if (store == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "门店不存在");
        }
        StoreDTO storeDTO = EntityUtil.changeEntityByOrika(StoreDTO.class, store);
        TabStoreGroup storeGroup = tabStoreGroupMapper.selectByPrimaryKey(storeId);
        if (storeGroup != null) {
            storeDTO.setStoreGroupName(storeGroup.getStoreGroupName());
        }
        // 门店照片
        List<TabStorePhoto> tabPhotoList = storePhotoService.listStorePhoto(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
        List<StorePhotoDTO> photoDTOList = EntityUtil.changeEntityListByOrika(StorePhotoDTO.class, tabPhotoList);
        storeDTO.setPhotoList(photoDTOList);
        // 营业时间
        List<TabStoreBusinessTime> tabBusinessTimeList = storeBusinessTimeService.listBusinessTime(store.getStoreId());
        List<StoreBusinessTimeDTO> businessTimeDTOList = EntityUtil.changeEntityListByOrika(StoreBusinessTimeDTO.class, tabBusinessTimeList);
        storeDTO.setBusinessTimeList(businessTimeDTOList);
        storeBusinessTimeService.convertBusinessTime(businessTimeDTOList);
        // 门店品牌
        List<TabStoreBrand> storeBrandList = storeBrandService.listStoreBrandByIds(store.getBrandIds());
        List<StoreBrandDTO> brandDTOList = EntityUtil.changeEntityListByOrika(StoreBrandDTO.class, storeBrandList);
        storeDTO.setBrandList(brandDTOList);

        // 自定义属性
        List<TabStoreExtend> extendList = storeExtendService.listByStoreId(enterpriseId, storeId);
        List<StoreExtendDTO> extendDTOList = storeExtendService.convertStoreExtendToDTO(extendList);
        storeDTO.setStoreExtendList(extendDTOList);

        return ServiceResponse.success(storeDTO);
    }

    @Override
    public ServiceResponse<Integer> costStoreMove(String storeIds, String toStoreId) {
        return null;
    }

    @Override
    public ServiceResponse<List<List<String>>> listImportTitle(Integer enterpriseId, Integer regionId) {
        List<List<String>> list = new ArrayList<>();

        String[] rowName = {"序号", "门店名称", "门店代码", "门店类型", "门店状态", "门店电话", "门店所属分组", "关联门店品牌", "省", "市", "区县", "门店地址（不包括省市区）", "经度", "纬度"};
        List<String> rowNameList = new ArrayList<>();
        Collections.addAll(rowNameList, rowName);

        String[] description = {"", "不能超过20个字", "必填，不能超过20个字符，为erp系统中门店唯一识别码", "门店类型描述", "门店状态描述", "门店电话描述", "门店所属分组描述", "用\"、\"隔开", "省描述", "市描述", "区县描述", "门店地址（不包括省市区）描述", "", ""};
        List<String> descriptionList = new ArrayList<>();
        Collections.addAll(descriptionList, description);

        list.add(rowNameList);
        list.add(descriptionList);

        List<TabStoreField> fields = storeFieldService.listStoreFieldByRegionId(regionId);
        if (CollectionUtils.isNotEmpty(fields)) {
            for (TabStoreField field : fields) {
                rowNameList.add(field.getStoreFieldName());
                String storeFieldDetail = field.getStoreFieldDetail();
                JSONObject jsonObject = JSON.parseObject(storeFieldDetail);
                descriptionList.add(jsonObject.getString("tipMessage"));
            }
        }

        return ServiceResponse.success(list);
    }
}
